"""(Skeleton) Library to fetch data from a Splinx API"""

import asyncio
import hmac
import hashlib
import time
from datetime import datetime
from pathlib import Path
from typing import Any

from pydantic import HttpUrl
from httpx import AsyncClient, URL

import pandas as pd  # type: ignore

from knoc.settings import settings
from knoc_splynx.models import (
    SplynxAuthDataModel,
    SplynxCustomerModel,
)


class KSplynxError(Exception):
    pass


class KSplynxLoginError(KSplynxError):
    pass


class SplynxAPI(AsyncClient):
    _splynx_domain: URL
    _api_key: str
    _api_secret: str
    _token_url: str = "admin/auth/tokens"
    __nonce_v: int
    _auth_data: SplynxAuthDataModel

    def __init__(self, url: URL | HttpUrl | str, key: str, sec: str, *args, **kwargs):
        self._splynx_domain = isinstance(url, URL) and url or URL(str(url))
        self._api_key = key
        self._api_secret = sec
        self.__nonce()
        super().__init__(*args, **kwargs)

    def _auth_request_data(self) -> dict:
        return {
            "auth_type": settings.splynx.authtypeapikey,
            "key": self._api_key,
            "nonce": self.__nonce_v,
            "signature": self.__signature(),
        }

    def __signature(self) -> str:
        st = "%s%s" % (self.__nonce_v, self._api_key)
        signature_hash = hmac.new(
            bytes(self._api_secret, "latin-1"), bytes(st, "latin-1"), hashlib.sha256
        ).hexdigest()
        return signature_hash.upper()

    def __nonce(self):
        time_now = datetime.now()
        self.__nonce_v = round(
            (time.mktime(time_now.timetuple()) + time_now.microsecond / 1000000.0) * 100
        )
        # self._debug_message("Nonce: {}".format(self.__nonce_v))

    # def _auth_request_data(self) -> dict[str, str]:
    #    auth_data = {
    #        "login": self._api_secret,
    #        "password": self._api_key,
    #    }
    #    auth_data["auth_type"] = "admin"
    #    return auth_data

    def make_url(self, url: str) -> URL:
        return self._splynx_domain.join(f"api/2.0/{url}")

    async def request(
        self,
        method: str,
        url: str,
        auth_data: SplynxAuthDataModel | None = None,
        *args,
        **kwargs,
    ):
        headers = kwargs.pop("headers")
        if auth_data is not None:
            self._auth_data = auth_data
        if hasattr(self, "_auth_data"):
            if headers is None:
                headers = {}
            headers["Authorization"] = (
                "Splynx-EA (access_token=" + str(self._auth_data.access_token) + ")"
            )
        return await super().request(
            method, url=self.make_url(url), headers=headers, *args, **kwargs
        )

    async def login(self, **kwargs):
        resp = await self.post(
            self._token_url, data=self._auth_request_data(), **kwargs
        )
        if not resp.is_success:
            raise KSplynxLoginError(f"Cannot login: {resp.json()["error"]["message"]}")
        self._auth_data = SplynxAuthDataModel.model_validate_json(resp.content)
        return self._auth_data

    async def logout(self):
        """
        Method for authorize on Splynx system API.

        This method make api call to Splynx system and deactivate API token.
        See more details about Splynx API authorization on
        page: https://splynx.docs.apiary.io/#introduction/authentication/by-access-token

        :return bool: Deactivate token result.
        """
        raise NotImplementedError()
        # response = self.delete(self._token_url)  # , entity_id=self.__refresh_token)
        # self.__access_token = None
        # self.__access_token_expiration = None
        # self.__refresh_token = None
        # self.__refresh_token_expiration = None
        # self.__permissions = None
        # return response

    async def get_customers(self) -> list[dict[str, str]]:
        """Get Splynx customers, flattening the fields in the extra fields
        with an "aa-" prefix"""
        resp = await splynx_api.get("admin/customers/customer")
        # await splynx_api.logout()
        resp_json = resp.json()
        customers_json = splynx_api.expand_splynx_aa(resp_json)
        # customers = SplynxCustomersModel.model_validate_json(resp.content)
        return customers_json

    def expand_splynx_aa(
        self,
        customers: list[dict[str, str | dict[str, str]]],
    ) -> list[dict[str, str]]:
        """Expand Splynx customer additional attributes to inline, prefixed with aa_"""
        for customer in customers:
            if aas := customer.get("additional_attributes"):
                assert isinstance(aas, dict)
                for key, value in aas.items():
                    customer[f"aa_{key}"] = value
                del customer["additional_attributes"]
        return customers  # type: ignore


splynx_api = SplynxAPI(settings.splynx.url, settings.splynx.key, settings.splynx.sec)


async def get_customers(pandas_export=True, file_name="splynx_customers.feather"):
    await splynx_api.login()
    customers_json = await splynx_api.get_customers()

    # if check_type:
    #    customers = {c["id"]: SplynxCustomerModel.model_validate(c) for c in resp_json}
    #    print(f"Found {len(customers)} customers")
    #    print(f"First one: {customers[customers_json[0]['id']]}")

    if pandas_export:
        df = pd.DataFrame.from_records(customers_json)
        df.to_feather(settings.data_path / file_name)
        return df


if __name__ == "__main__":
    asyncio.run(get_customers())
