from pydantic import BaseModel

try:
    import pandas as pd  # type: ignore
except ImportError:
    pd = None
from hatchet_sdk import Context

from knoc import __version__
from knoc.settings import settings
from knoc.db import aio_df_to_sql
from knoc.hatchet_client import hatchet
from knoc_splynx.ksplynx import splynx_api, SplynxCustomerModel, KSplynxError
from knoc.workflows.utils import ImportReportRecordType, ImportReport


class SplynxImportParams(BaseModel):
    to_db: bool = True
    to_feather: bool = False
    check_types: bool = False


import_splynx_customers_wf = hatchet.workflow(
    name="Import-Splynx-Customers",
    input_validator=SplynxImportParams,
    version=__version__,
)


@import_splynx_customers_wf.task()
async def import_splynx_customers(
    input: SplynxImportParams,
    ctx: Context,
) -> ImportReport:
    if pd is None:
        raise KSplynxError("Pandas is not installed")
    await splynx_api.login()
    resp = await splynx_api.get("admin/customers/customer")
    # await splynx_api.logout()
    resp_json = resp.json()
    # Expand Splynx customer additional attributes to inline, prefixed with aa_
    customers_json = splynx_api.expand_splynx_aa(resp_json)
    # customers = SplynxCustomersModel.model_validate_json(resp.content)

    if input.check_types:
        customers = {c["id"]: SplynxCustomerModel.model_validate(c) for c in resp_json}
        assert isinstance(customers, SplynxCustomerModel)

    df = pd.DataFrame.from_dict(customers_json)  # type: ignore

    if input.to_db:
        ctx.log(f"Write to DB {len(df)} records")
        await aio_df_to_sql(
            df,
            settings.splynx.db_table_customers,
            schema=settings.splynx.db_schema,
            if_exists="replace",
        )

    if input.to_feather:
        df.to_feather(settings.data_path / "customers.feather")

    return ImportReport(
        types=[ImportReportRecordType(name="Splynx customers", count=len(df))]
    )
